package hu.elte.inf.teralg.graph;

import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.Scanner;
import hu.elte.inf.teralg.MyQueue;

/**
 * Térinformatikai algoritmusok
 * 4. beadandó: Szélességi bejárás
 * @author Máté Cserép <mcserep@inf.elte.hu>
 */
public class BreadthFirstSearch {
    // Maximális távolság (1 átszállás == 2)
    private static final int MAX_DISTANCE = 2;

    public static void main(String[] args) {
        int n = 0, e = 0;          // csúcsok és élek száma
        String[] cities = null;    // városok (csúcsok) neve
        boolean[][] edges = null;  // csúcsmátrix

        // Adatok beolvasása
        try
        {
            FileReader fr = new FileReader("flights.txt");
            Scanner sc = new Scanner(fr);

            n = sc.nextInt();
            cities = new String[n];
            edges = new boolean[n][n];

            // Városok beolvasása és rendezése a későbbi bináris keresés lehetővé tételéhez
            for(int i = 0; i < n; ++i) {
                cities[i] = sc.next();
            }
            Arrays.sort(cities);

            // Éllista beolvasása
            e = sc.nextInt();
            for(int i = 0; i < e; ++i) {
                String from = sc.next();
                String to = sc.next();

                int fromIndex = Arrays.binarySearch(cities, from);
                int toIndex = Arrays.binarySearch(cities, to);
                edges[fromIndex][toIndex] = true;
                edges[toIndex][fromIndex] = true;
            }
            sc.close();
            fr.close();
        }
        catch(IOException ex) {
            System.out.println("File handling error: " + ex.getMessage());
        }

        if(cities.length == 0) {
            System.out.println("Ües bemenet.");
        }
        else if(Arrays.binarySearch(cities, "Budapest") < 0) {
            System.out.println("Budapest nincs a városok között.");
        }
        else {
            MyQueue<Integer> process = new MyQueue<Integer>();  // kiterjesztendő csúcsok sora
            boolean[] reached = new boolean[n];                 // már elért (sorba rakott) csúcsok nyilvántartása
            int[] distance = new int[n];                        // távolság nyilvántartása

            // Start csúcs inicializálása
            int start = Arrays.binarySearch(cities, "Budapest");
            process.push(start);
            reached[start] = true;
            distance[start] = 0;

            // Iteratív kiterjesztés
            System.out.println("Elérhető városok:");
            while(!process.empty()) {
                // Aktuálisan kiterjesztendő csúcs
                int current = process.pop();
                // Megállási feltétel, ha elértük a kívánt távolságot
                if(distance[current] > MAX_DISTANCE) break;

                System.out.println(cities[current]);
                for(int to = 0; to < n; ++to)
                    if(edges[current][to] && !reached[to]) {
                        process.push(to);
                        reached[to] = true;
                        distance[to] = distance[current] + 1;
                    }
            }
        }
    }
}
